@@ -51,7 +51,6 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
     This is a widget with tabs where each one is a ClientWidget
     """
 
-    # This is required for the new API
     NAME = 'ipython_console'
     REQUIRES = [Plugins.Application, Plugins.Console, Plugins.Preferences]
     OPTIONAL = [
@@ -306,6 +305,16 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
             ]
         }
 
+        self.pyw_editor_run_configuration = {
+            'origin': self.NAME,
+            'extension': 'pyw',
+            'contexts': [
+                {'name': 'File'},
+                {'name': 'Cell'},
+                {'name': 'Selection'},
+            ]
+        }
+
         self.executor_configuration = [
             {
                 'input_extension': 'py',
@@ -323,6 +332,14 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
                 'requires_cwd': True,
                 'priority': 0
             },
+            {
+                'input_extension': 'pyw',
+                'context': {'name': 'File'},
+                'output_formats': [],
+                'configuration_widget': IPythonConfigOptions,
+                'requires_cwd': True,
+                'priority': 0
+            },
             {
                 'input_extension': 'py',
                 'context': {'name': 'Cell'},
@@ -339,6 +356,14 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
                 'requires_cwd': True,
                 'priority': 0
             },
+            {
+                'input_extension': 'pyw',
+                'context': {'name': 'Cell'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 0
+            },
             {
                 'input_extension': 'py',
                 'context': {'name': 'Selection'},
@@ -355,6 +380,14 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
                 'requires_cwd': True,
                 'priority': 0
             },
+            {
+                'input_extension': 'pyw',
+                'context': {'name': 'Selection'},
+                'output_formats': [],
+                'configuration_widget': None,
+                'requires_cwd': True,
+                'priority': 0
+            },
             {
                 'input_extension': 'pyx',
                 'context': {'name': 'File'},
@@ -492,7 +525,8 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
         for run_config in [
             self.python_editor_run_configuration,
             self.ipython_editor_run_configuration,
-            self.cython_editor_run_configuration
+            self.cython_editor_run_configuration,
+            self.pyw_editor_run_configuration
         ]:
             editor.add_supported_run_configuration(run_config)
 
@@ -588,7 +622,8 @@ class IPythonConsole(SpyderDockablePlugin, RunExecutor):
         for run_config in [
             self.python_editor_run_configuration,
             self.ipython_editor_run_configuration,
-            self.cython_editor_run_configuration
+            self.cython_editor_run_configuration,
+            self.pyw_editor_run_configuration
         ]:
             editor.remove_supported_run_configuration(run_config)
 
